#Note: onAdd, onDelete, and onClone are propagated
#by manage_afterAdd, manage_beforeDelete, manage_afterClone respectively
#If either of these are overridden for a ObjectManager, then the overriding method is responsible
#for propagation of the appropriate onX methods.

#onPropertyChange

from ZClasses.Property import ZInstanceSheet
from OFS.PropertySheets import FixedSchema

def _updateProperty(self, id, value, meta=None):
    old = self.getProperty(id)
    if old == value:
        return
    change = {'id':id,'old':old,'new':value,'change':'change'}
    FixedSchema._updateProperty(self, id, value, meta)
    self._v_changes[id] = change
        
def manage_editProperties(self, REQUEST):
    """Edit object properties via the web.  Any undefined properties in REQUEST
       will be marked blank (ie. checkboxes, textboxes)
    """
    self._v_changes = {}
    result = FixedSchema.manage_editProperties(self, REQUEST)
    o = self.v_self()
    if len(self._v_changes) > 0 and hasattr(o.aq_base, 'onPropertyChange'):
        o.onPropertyChange(self.getId(), self._v_changes, self.REQUEST)
    return result
    
def manage_changeProperties(self, REQUEST=None, **kw):
    """Change object properties."""
    self._v_changes = {}
    result = apply(FixedSchema.manage_changeProperties,(self,REQUEST), kw)
    o = self.v_self()
    if len(self._v_changes) > 0 and hasattr(o.aq_base, 'onPropertyChange'):
        o.onPropertyChange(self.getId(), self._v_changes, self.REQUEST)
    return result

ZInstanceSheet._updateProperty = _updateProperty
ZInstanceSheet.manage_editProperties = manage_editProperties
ZInstanceSheet.manage_changeProperties = manage_changeProperties

# onAdd and onDelete

from OFS.ObjectManager import ObjectManager
from Acquisition import aq_base

ObjectManager._setObject2 = ObjectManager._setObject
ObjectManager._delObject2 = ObjectManager._delObject

def _setObject(self,id,object,roles=None,user=None, set_owner=1):
    id = ObjectManager._setObject2(self,id,object,roles,user,set_owner)
    object = self._getOb(id)
    if hasattr(aq_base(object), 'onAdd'):
        object.onAdd(self.REQUEST)
    return id
        
def _delObject(self, id, dp=1):
    object = self._getOb(id)
    if hasattr(aq_base(object), 'onDelete'):
        object.onDelete(self.REQUEST)
    ObjectManager._delObject2(self, id, dp)

def manage_afterAdd(self, item, container):
    for object in self.objectValues():
        try: s=object._p_changed
        except: s=0
        if hasattr(aq_base(object), 'manage_afterAdd'):
            object.manage_afterAdd(item, container)
        if hasattr(aq_base(object), 'onAdd'):
            object.onAdd(self.REQUEST)
        if s is None: object._p_deactivate()

def manage_beforeDelete(self, item, container):
    for object in self.objectValues():
        try: s=object._p_changed
        except: s=0
        try:
            if hasattr(aq_base(object), 'manage_beforeDelete'):
                object.manage_beforeDelete(item, container)
            if hasattr(aq_base(object), 'onDelete'):
                object.onDelete(self.REQUEST)
        except BeforeDeleteException, ob:
            raise
        except:
            LOG('Zope',ERROR,'manage_beforeDelete() threw',
                error=sys.exc_info())
            pass
        if s is None: object._p_deactivate()

def manage_afterClone(self, item):
    if self is item and hasattr(aq_base(self), 'onClone'):
        self.onClone(self.REQUEST)
    for object in self.objectValues():
        try: s=object._p_changed
        except: s=0
        if hasattr(aq_base(object), 'manage_afterClone'):
            object.manage_afterClone(item)
        if hasattr(aq_base(object), 'onClone'):
            object.onClone(self.REQUEST)
        if s is None: object._p_deactivate()

ObjectManager._setObject = _setObject
ObjectManager._delObject = _delObject
ObjectManager.manage_afterAdd = manage_afterAdd
ObjectManager.manage_beforeDelete = manage_beforeDelete
ObjectManager.manage_afterClone = manage_afterClone